from .subviewparent import SubViewParent



class Sheet (SubViewParent):
    """A """
    def __init__(self, presented:bool=False) -> None:
        super().__init__()

        self.presented = presented

        self.vdata.update({
            "name" : "Sheet",
            "props" : {
                "presented" : presented
            }
        })
    

    def update(self):
        self.vdata.update({
            "name" : "Sheet",
            "props" : {
                "presented" : self.presented
            }
        })
        return super().update()


from .subviewparent import SubViewParent
from ..tools.on_action import on_view_action


class GeometryReader (SubViewParent):
    """A container view that defines its content as a 
    function of its own size and coordinate space.
    
    You can use this to get the frame size (width & height) and check resizing event.
    """
    def __init__(self, on_resize=None) -> None:
        super().__init__()

        self.__width = None
        self.__height = None

        self.on_resize = on_resize

        self.vdata.update({
            "name" : "GeometryReaderView"
        })
    

    def on_event (self, event_content:dict):
        if event_content['name'] == "on_resize":
            self.__width = int(event_content['width'])
            self.__height = int(event_content['height'])
            on_view_action(self.on_resize, [self])

    @property
    def width (self):
        return self.__width
    
    @property
    def height (self):
        return self.__height


import sys



def is_device_a_mac ():
    if "darwin" in str(sys.platform):
        return True
    else:
        return False


def is_device_a_ios ():
    if "ios" in str(sys.platform):
        return True
    else:
        return False


from .subview import SubView




class Icon (SubView):
    """
    A standard label for user interface items, consisting of an icon with a title.
    
    You can browse all the available icons on:
    [Apple SF symbols](https://developer.apple.com/sf-symbols/)
    """
    def __init__(self, icon_name:str="globe.europe.africa", icon_color:str="primery", icon_size:int=18) -> None:
        super().__init__()

        self.icon_name : str = icon_name
        self.icon_color : str = icon_color
        self.icon_size : int = icon_size

        self.vdata.update({
            "name" : "Icon",
            "props" : {
                "icon_name" : self.icon_name,
                "icon_color" : self.icon_color,
                "icon_size" : self.icon_size
            }
        })
    
    def update(self):
        self.vdata.update({
            "name" : "Icon",
            "props" : {
                "icon_name" : self.icon_name,
                "icon_color" : self.icon_color,
                "icon_size" : self.icon_size
            }
        })
        return super().update()


from .subview import SubView
from ..tools.on_action import on_view_action


class ElevatedButton (SubView):
    """A button with a text and background color."""
    def __init__(self, content:str, color:str="white", bgcolor:str="blue", width:int=80, height:int=40, corner_radius:int=12, on_click=None) -> None:
        super().__init__()

        self.content : str = content
        self.color : str = color
        self.bgcolor : str = bgcolor
        self.width : int = width
        self.height : int = height
        self.corner_radius : int = corner_radius
        self.on_click = on_click

        self.vdata.update({
            "name" : "ElevatedButton"
        })

        self.vdata['props'].update({
            "content" : self.content,
            "color" : self.color,
            "bgcolor" : self.bgcolor,
            "width" : self.width,
            "height" : self.height,
            "corner_radius" : self.corner_radius
        })
    

    def on_event (self, event_content:dict):
        if event_content['name'] == "on_click":
            on_view_action(self.on_click, [self])

    
    def update(self):
        self.vdata['props'].update({
            "content" : self.content,
            "color" : self.color,
            "bgcolor" : self.bgcolor,
            "width" : self.width,
            "height" : self.height,
            "corner_radius" : self.corner_radius
        })
        return super().update()


"""
swoopyui is a python library that allow you to build swoopyui applications in python only!

Visit the github page:
[swoopyui github](https://github.com/SKbarbon/swoopyui)
"""
from .tools.check_platform import is_device_a_ios, is_device_a_mac
if is_device_a_mac() or is_device_a_ios():
    pass
else:
    print("WARNING: This platform is not supported for the development.")
    # raise Exception ("Your device is not a mac. So cannot use this library.")

from .swoopyui import app

from .view import View

from .views.text import Text
from .views.vstack import VStack
from .views.hstack import HStack
from .views.navigationsplitview import NavigationSplitView
from .views.button import Button
from .views.navigationlink import NavigationLink
from .views.textfield import TextField
from .views.scrollview import ScrollView, ScrollModeHorizontal, ScrollModeVertical
from .views.colorview import ColorView
from .views.spacer import Spacer
from .views.list import List
from .views.sheet import Sheet
from .views.elevatedbutton import ElevatedButton
from .views.textbutton import TextButton
from .views.label import Label
from .views.icon import Icon
from .views.contextmenu import ContextMenu
from .views.labeledbutton import LabeledButton
from .views.menu import Menu
from .views.scrollingtabview import ScrollingTabView
from .views.navigationstack import NavigationStack
from .views.videoplayer import VideoPlayer
from .views.geometryreader import GeometryReader
from .views.zstack import ZStack
# from .views.image import Image
# from .views.webview import WebView
# from .views.animatedview import AnimatedView

# shapes views
from .views.shapes.roundedrectangleshape import RoundedRectangleShape



from .subviewparent import SubViewParent



class ContextMenu (SubViewParent):
    """A container for views that you present as menu items in a context menu."""
    def __init__(self) -> None:
        super().__init__()

        self.vdata.update({
            "name" : "ContextMenu"
        })

    
    def add_to_context_menu (self, subviews:list):
        """Add a new subview to the context menu."""
        if self.parent_view == None or self.main_view == None: return

        for subview in subviews:
            subview.respown(
                new_id=self.main_view.get_new_subview_id(),
                update_id=self.main_view.get_new_subview_update_id(),
                parent_view=self,
                main_view=self.main_view
            )
            self.subviews.append(subview)
            self.vdata['sub_views2'].append(subview.vdata)

            self.main_view.add_to_subviews_history(subview)
        self.update()


from .subviewparent import SubViewParent
from .subview import SubView



class NavigationSplitView (SubViewParent):
    """A view that presents views in two or three columns, where selections in 
    leading columns control presentations in subsequent columns."""
    def __init__(self, title:str="", detail_title:str="") -> None:
        super().__init__()

        self.title : str = title
        self.detail_title : str = detail_title

        self.vdata.update({
            "name" : "NavigationSplitView",
            "sub_views2" : [],
            "sub_views3" : [],
            "sub_views4" : []
        })

        self.vdata['props'].update({
            "title" : title,
            "detail_title" : detail_title
        })
    
    def add_to_detail (self, subviews:list):
        """Add new subview to the second column of the navigationview."""
        if self.parent_view == None or self.main_view == None: return

        for subview in subviews:
            subview.respown(
                new_id=self.main_view.get_new_subview_id(),
                update_id=self.main_view.get_new_subview_update_id(),
                parent_view=self,
                main_view=self.main_view
            )
            self.subviews.append(subview)
            self.vdata['sub_views2'].append(subview.vdata)
            self.main_view.add_to_subviews_history(subview)
        self.update()
    
    def add_to_toolbar (self, subviews:list):
        """Add subviews in toolbar."""
        if self.parent_view == None or self.main_view == None: return

        for subview in subviews:
            subview.respown(
                new_id=self.main_view.get_new_subview_id(),
                update_id=self.main_view.get_new_subview_update_id(),
                parent_view=self,
                main_view=self.main_view
            )
            self.subviews.append(subview)
            self.vdata['sub_views3'].append(subview.vdata)
            self.main_view.add_to_subviews_history(subview)
        self.update()
    

    def add_to_detail_toolbar (self, subviews:list):
        """Add to detail page toolbar."""
        if self.parent_view == None or self.main_view == None: return

        for subview in subviews:
            subview.respown(
                new_id=self.main_view.get_new_subview_id(),
                update_id=self.main_view.get_new_subview_update_id(),
                parent_view=self,
                main_view=self.main_view
            )
            self.subviews.append(subview)
            self.vdata['sub_views4'].append(subview.vdata)
            self.main_view.add_to_subviews_history(subview)
        self.update()
    
    def update(self):
        self.vdata['props'].update({
            "title" : self.title,
            "detail_title" : self.detail_title
        })
        return super().update()


from .subview import SubView




class SubViewParent (SubView):
    """The `SubViewParent` is a subview that have the abiliy to add another subviews inside it."""
    def __init__(self) -> None:
        super().__init__()

        self.subviews = []
        self.vdata.update({
            "sub_views" : [],
            "sub_views2" : [],
            "sub_views3" : [],
            "sub_views4" : []
        })
    

    def add (self, subviews:list):
        """Add a new subview to this view."""
        if self.parent_view == None or self.main_view == None: return

        for subview in subviews:
            subview.respown(
                new_id=self.main_view.get_new_subview_id(),
                update_id=self.main_view.get_new_subview_update_id(),
                parent_view=self,
                main_view=self.main_view
            )
            self.subviews.append(subview)
            self.vdata['sub_views'].append(subview.vdata)

            self.main_view.add_to_subviews_history(subview)
        self.update()

    def update(self):
        return super().update()


from .views.subview import SubView



class onAddNewSubViewRequest:
    def __init__(self, subview:SubView) -> None:
        if subview.parent_view == None: return

        to_id = ""
        if subview.parent_view == subview.main_view:
            to_id = "main"
        else:
            to_id = str(subview.parent_view.id)

        self.raw = {
            "action" : "add_subview",
            "to_id" : to_id,
            "subview_data" : subview.vdata
        }


class onUpdateSubviewProps:
    def __init__(self, subview:SubView) -> None:
        self.raw = {
            "action" : "update_subview",
            "subview_data" : subview.vdata
        }
    


from flask import request, Flask



def swoopyui_host_setup (flask_app:Flask, target_function, client_view):
    @flask_app.route("/start_target_function")
    def start_target_function ():
        target_function(client_view)
        return '{"ok":true}'
    
    @flask_app.route("/get_updates")
    def get_updates():
        updts = list(client_view.next_updates)
        client_view.next_updates.clear()
        print(updts)
        return {
            "updts" : updts
        }
    
    @flask_app.route("/push_update")
    def push_update():
        return ""






class SubView (object):
    """The parent class of all subview's subclassess."""
    def __init__(self) -> None:
        self.__id = -1
        self.__parent_view = None
        self.__main_view = None

        self.update_id = -1

        self.vdata = {
            "ID" : self.__id,
            "update_id" : self.update_id,
            "name" : "None",
            "props" : {},
            "sub_views" : []
        }

    
    def respown (self, new_id:int, update_id:int, parent_view, main_view):
        self.__id = new_id
        self.update_id = update_id
        self.__parent_view = parent_view
        self.__main_view = main_view

        self.vdata['ID'] = new_id
        self.vdata['update_id'] = update_id
    
    def update (self):
        if self.parent_view == None or self.main_view == None:
            raise Exception("You must add this subview to the UI first.")
        self.vdata['ID'] = self.__id
        self.vdata['update_id'] = self.update_id

        self.main_view.update(self)


    @property
    def id (self):
        return self.__id
    
    @property
    def parent_view (self):
        return self.__parent_view
    
    @property
    def main_view(self):
        return self.__main_view






def is_run_on_pyinstaller ():
    import sys

    if getattr(sys, 'frozen', False):
        # Running from a PyInstaller executable
        return True
    else:
        # Running the script directly
        return False


import os, tempfile



def prepare_swiftUI_for_ios (port):
    documents_dir_path = os.path.expanduser('~/Documents')
    
    if not os.path.isdir(os.path.join(documents_dir_path, "swoopyui_temp")):
        os.mkdir(os.path.join(documents_dir_path, "swoopyui_temp"))
    
    swoopyui_connect_file_path = os.path.join(documents_dir_path, "swoopyui_temp", "swoopyui_connect.txt")
    open(swoopyui_connect_file_path, "w+", encoding="utf-8").write(f"{port}")


from .subview import SubView



class VideoPlayer (SubView):
    """A view that displays content from a player and a native user interface to control playback."""
    def __init__(self, link:str, width:int=250, height:int=250) -> None:
        super().__init__()

        self.link : str = link
        self.width : int = width
        self.height : int = height

        self.vdata.update({
            "name" : "AV_VideoPlayer",
            "props" : {
                "link" : link,
                "width" : self.width,
                "height" : self.height
            }
        })
    

    def play (self):
        """Begins playback of the current video."""
        if self.main_view != None:
            self.main_view.subview_reverse_event({
                "action" : "change_video_state",
                "new_video_state" : "play",
                "parent_id" : self.id
            })
    
    
    def puase (self):
        """Pauses playback of the current video."""
        if self.main_view != None:
            self.main_view.subview_reverse_event({
                "action" : "change_video_state",
                "new_video_state" : "puase",
                "parent_id" : self.id
            })
    

    def change_video_link (self, new_link:str, same_position:bool):
        """Replace and change the current video link with a new one without pushing a new subview update.
        
        This is usefull when you just want to replace the video with an edited version, or for example when you just want to change a
        high-quality video link with a low-quality video link.

        the `same_position` arguments make the video replace the link, then continue playing the video from the same last duration position.
        """
        self.link = str(new_link)
        self.vdata['props']['link'] = str(new_link)
        if self.main_view != None:
            self.main_view.subview_reverse_event({
                "action" : "change_video_link",
                "new_link" : str(new_link),
                "same_position" : bool(same_position),
                "parent_id" : self.id
            })
    
    def change_videoplayer_bounds (self, width:int, height:int):
        """Change the video player width and height without pushing a new update 
        event for this subview"""
        self.width = int(width)
        self.height = int(height)
        self.vdata['props'].update({
            "width" : int(width),
            "height" : int(height)
        })
        if self.main_view != None:
            self.main_view.subview_reverse_event({
                "action" : "resize",
                "width" : int(width),
                "height" : int(height),
                "parent_id" : self.id
            })


from .subview import SubView



class Text (SubView):
    def __init__ (self, content:str, color:str="primery",  size:int=18, bold:bool=False):
        super().__init__()

        self.content : str = content
        self.color : str = color
        self.size : str = size
        self.bold : bool = bold

        self.vdata.update({
            "name" : "Text",
            "props" : {
                "content" : self.content,
                "color" : self.color,
                "size" : self.size,
                "bold" : self.bold
            }
        })

    
    def update (self):
        self.vdata.update({
            "name" : "Text",
            "props" : {
                "content" : self.content,
                "color" : self.color,
                "size" : self.size,
                "bold" : self.bold
            }
        })
        super().update()


from .subviewparent import SubViewParent



class Menu (SubViewParent):
    """A control for presenting a menu of actions."""
    def __init__(self, title:str="") -> None:
        super().__init__()


        self.title : str = title

        self.vdata.update({
            "name" : "Menu",
            "props" : {
                "title" : self.title
            }
        })
    
    def update(self):
        self.vdata.update({
            "name" : "Menu",
            "props" : {
                "title" : self.title
            }
        })
        return super().update()


from .subviewparent import SubViewParent
from ..tools.on_action import on_view_action


class Button (SubViewParent):
    """A button with subviews inside it"""
    def __init__(self, width:float=100, height:float=40, corner_radius:float=12, on_click=None) -> None:
        super().__init__()

        self.vdata.update({
            "name" : "Button"
        })

        self.width : float = width
        self.height : float = height
        self.corner_radius : float = corner_radius
        self.on_click = on_click

        self.vdata['props'].update({
            "width" : self.width,
            "height" : self.height,
            "corner_radius" : self.corner_radius
        })
    

    def on_event (self, event_content:dict):
        if event_content['name'] == "on_click":
            on_view_action(self.on_click, [self])

    def update(self):
        self.vdata['props'].update({
            "width" : self.width,
            "height" : self.height,
            "corner_radius" : self.corner_radius
        })
        return super().update()


from .subviewparent import SubViewParent
from .subview import SubView


class NavigationLink (SubViewParent):
    def __init__(self) -> None:
        super().__init__()

        self.vdata.update({
            "name" : "NavigationLink",
            "sub_views2": []
        })
    
    def add_to_destination (self, subviews:list):
        """Add new subview to the navigated destination."""
        if self.parent_view == None or self.main_view == None: return

        for subview in subviews:
            subview.respown(
                new_id=self.main_view.get_new_subview_id(),
                update_id=self.main_view.get_new_subview_update_id(),
                parent_view=self,
                main_view=self.main_view
            )
            self.subviews.append(subview)
            self.vdata['sub_views2'].append(subview.vdata)
            self.main_view.add_to_subviews_history(subview)
            self.update()


from .subviewparent import SubViewParent



class List (SubViewParent):
    """Align the subviews in a parent stack verticaly."""
    def __init__(self) -> None:
        super().__init__()

        self.vdata.update({
            "name" : "TheListView"
        })


from .subviewparent import SubViewParent
from .subview import SubView



class NavigationStack (SubViewParent):
    """A view that displays a root view and enables you 
    to present additional views over the root view."""
    def __init__(self, title:str="") -> None:
        super().__init__()

        self.title : str = title


        self.vdata.update({
            "name" : "NavigationStack"
        })

        self.vdata['props'].update({
            "title" : title
        })
    
    def update(self):
        self.vdata['props'].update({
            "title" : self.title
        })
        return super().update()

    def add_to_toolbar (self, subviews:list):
        """Add a subview to the navigation toolbar"""
        if self.parent_view == None or self.main_view == None: return

        for subview in subviews:
            subview.respown(
                new_id=self.main_view.get_new_subview_id(),
                update_id=self.main_view.get_new_subview_update_id(),
                parent_view=self,
                main_view=self.main_view
            )
            self.subviews.append(subview)
            self.vdata['sub_views2'].append(subview.vdata)
            self.main_view.add_to_subviews_history(subview)
        self.update()


from .subviewparent import SubViewParent




class ScrollingTabView (SubViewParent):
    """A scrolling tabview for multi-page on scroll"""
    def __init__(self) -> None:
        super().__init__()


        self.vdata.update({
            "name" : "ScrollingTabView"
        })
    
    def navigate_to_subview (self, subview_id:int):
        if self.main_view != None:
            self.main_view.subview_reverse_event({
                "action" : "tabview_selection",
                "parent_id" : self.id,
                "subview_id" : subview_id
            })


from .subview import SubView
from ..tools.on_action import on_view_action



class TextField (SubView):
    def __init__(self, placeholder:str="", content:str="", resizeable:bool=False, width:int=100, height:int=40, color:str="primery", on_change=None) -> None:
        super().__init__()

        self.placeholder = placeholder
        self.content : str = content
        self.resizeable : bool = resizeable
        self.width : int = width
        self.height : int = height
        self.color : int = color
        self.on_change = on_change

        self.vdata.update({
            "name" : "TextField",
            "props" : {
                "content" : self.content,
                "placeholder" : self.placeholder,
                "resizeable" : self.resizeable,
                "width" : width,
                "height" : height,
                "color" : self.color
            }
        })
    
    
    def on_event (self, event_content:dict):
        if event_content['name'] == "on_change":
            self.content = str(event_content['content'])
            self.vdata['props']['content'] = str(event_content['content'])
            on_view_action(self.on_change, [self])

    def update (self):
        self.vdata.update({
            "name" : "TextField",
            "props" : {
                "content" : self.content,
                "placeholder" : self.placeholder,
                "resizeable" : self.resizeable,
                "width" : self.width,
                "height" : self.height,
                "color" : self.color
            }
        })
        super().update()


import traceback





def run_the_target (target_function, function_args):
    try:
        target_function(*function_args)
    except Exception as e:
        traceback.print_exc()
        raise Exception(e)


import swoopyui

def main (view:swoopyui.View):
    def on_c (cls:swoopyui.ElevatedButton):
        print("Here!")
        testing_label.content = str("No its two!")
        testing_label.update()
    
    nav = swoopyui.NavigationStack("Try effect!")
    view.add(nav)

    nl = swoopyui.NavigationLink()
    nav.add([nl])

    nl.add([
        swoopyui.Text("Go to testing page!")
    ])

    testing_label = swoopyui.Text("One?")
    vt = swoopyui.VStack()
    nl.add_to_destination([vt])

    vt.add([
        testing_label,
        swoopyui.ElevatedButton("Change", on_click=on_c)
    ])

    nl.update()


from .views.subview import SubView
from .protocol import onAddNewSubViewRequest, onUpdateSubviewProps


class View (object):
    def __init__(self, host_port, host_app_class) -> None:
        self.__host_app_class = host_app_class
        self.host_port = host_port

        self.platform = None #? The platform that running swiftUI.
        self.theme_mode = None #? The theme mode of the app, like light mode or dark mode.

        self.__subviews_history = [] #? All the subviews that have been added directly or not directly on currently connected swiftUI app.
        self.__last_id = 0
        self.__last_update_id = 0
        
        self.subviews = []
    
    def add (self, subview:SubView):
        """Add a new subview to the main view."""
        subview.respown(
            new_id=self.get_new_subview_id(),
            update_id=self.get_new_subview_update_id(),
            parent_view=self,
            main_view=self
        )
        self.subviews.append(subview)
        self.__subviews_history.append(subview)
        
        self.__add_to_next_update_requests(onAddNewSubViewRequest(subview=subview).raw)
        subview.update()
    
    def remove (self, subview:SubView):
        """Remove a subview from the swiftUI app."""
    
    def get_subview_by_id (self, subview_id:int):
        """This function will search for the subviewID in the subviews history.
        
        If there is no result it will return `None`.
        """
        for sbv in self.__subviews_history:
            if sbv.id == subview_id:
                return sbv
        return None
    

    def get_new_subview_id (self):
        self.__last_id = self.__last_id + 1
        return self.__last_id
    
    def get_new_subview_update_id (self):
        self.__last_update_id = self.__last_update_id + 1
        return self.__last_update_id

    
    def update (self, subview:SubView=None):
        if subview == None:
            pass
        else:
            subview.update_id = self.get_new_subview_update_id()
            subview.vdata['update_id'] = subview.update_id
            self.__add_to_next_update_requests(update_dict=onUpdateSubviewProps(subview=subview).raw)
    

    def add_to_subviews_history (self, subv:SubView):
        self.__subviews_history.append(subv)
    
    
    def process_client_events (self, event_dict:dict):
        if event_dict['action'] == "view_event":
            subview_id = int(event_dict['view_id'])
            subview_class = None
            for sbv in self.__subviews_history:
                sbv : SubView = sbv
                if int(sbv.id) == int(subview_id):
                    subview_class = sbv
                    break
            subview_class.on_event(event_dict['event_content'])
        
        elif event_dict['action'] == "startup_app_info":
            self.platform = event_dict['content']['platform']


    def subview_reverse_event (self, update_dict):
        """It when the subview request an event from the python side into the swift side."""
        self.__add_to_next_update_requests(update_dict=update_dict)


    def __add_to_next_update_requests(self, update_dict):
        """Add an update request to the client."""
        # print(update_dict)
        self.__host_app_class.next_get_updates_responses.append(update_dict)
    

    @property
    def next_updates (self):
        return self.__next_updates


from .subviewparent import SubViewParent



ScrollModeVertical = "vertical"
ScrollModeHorizontal = "horizontal"


class ScrollView (SubViewParent):
    """"""
    def __init__(self, scrollMode:ScrollModeVertical="vertical") -> None:
        super().__init__()

        self.scrollMode = scrollMode

        self.vdata.update({
            "name" : "ScrollView"
        })

        self.vdata['props'].update({
            "scrollMode" : self.scrollMode
        })
    

    def update(self):
        self.vdata['props'].update({
            "scrollMode" : self.scrollMode
        })
        return super().update()


from .tools.swoopyui_host import swoopyui_host_setup
from .tools.check_platform import is_device_a_ios, is_device_a_mac
from .tools.run_target import run_the_target
from .tools.run_swiftUI_ios import prepare_swiftUI_for_ios
from .tools.run_swiftUI import run_swiftUI_app
from .tools.get_free_port import get_free_port
from .view import View
from flask import Flask, request
import logging, threading, tempfile, time, shutil, os



class app:
    def __init__(self, target, base_name="__main__", debug:bool=False, for_preview=False) -> None:
        self.target_function = target
        self.base_name = base_name
        self.debug = debug
        self.for_preview = for_preview

        self.host_port = get_free_port()
        self.next_get_updates_responses = []

        self.client_view = View(host_port=self.host_port, host_app_class=self)

        # start the host
        self.tmp_dir = ""
        print("Host started..")
        self.host()

    def host (self):
        self.flask_app = Flask(self.base_name)
        flask_app = self.flask_app

        # Set the logging level to ignore warnings
        if self.debug == False and self.for_preview == False:
            log = logging.getLogger('werkzeug')
            log.setLevel(logging.ERROR)
        
        @flask_app.route("/start_target_function")
        def start_target_function ():
            print("A swiftUI client connected..")
            threading.Thread(target=run_the_target, args=[self.target_function, [self.client_view]]).start()
            return '{"ok":true}'
        
        @flask_app.route("/get_updates")
        def get_updates():
            updts = list(self.next_get_updates_responses)
            self.next_get_updates_responses.clear()
            return {
                "updts" : updts
            }
        
        @flask_app.route("/push_update", methods=['POST'])
        def push_update():
            self.client_view.process_client_events(request.json)
            return ""
            
        @flask_app.route('/shutdown')
        def shutdown():
            """Close the swoopyui host and delete the client app."""
            if os.path.isdir(self.tmp_dir):
                shutil.rmtree(self.tmp_dir)
                if self.debug:
                    print("swoopyui tmp dir deleted!")
                    print("host will close now!")
            
            os._exit(0)
            return 'Server shutting down...'


        if is_device_a_ios() and self.for_preview == False:
            prepare_swiftUI_for_ios(port=self.host_port)
        
        elif is_device_a_mac() and self.for_preview == False:
            tmp_dir = tempfile.mkdtemp()
            self.tmp_dir = tmp_dir
            threading.Thread(target=run_swiftUI_app, args=[self.host_port, tmp_dir], daemon=True).start()

        if self.for_preview:
            print("Your preview host ready, this is the host data:\n\n")
            threading.Thread(target=self.no_logging_after_3_sec).start()
            flask_app.run(host="0.0.0.0", port=self.host_port)
        else:
            flask_app.run(host="localhost", port=self.host_port)
    
    def no_logging_after_3_sec (self):
        time.sleep(3)
        log = logging.getLogger('werkzeug')
        log.setLevel(logging.ERROR)


from ..tools.on_action import on_view_action
from .subview import SubView


class TextButton (SubView):
    """A button with a text only."""
    def __init__(self, content:str, on_click=None) -> None:
        super().__init__()

        self.content : str = content
        self.on_click = on_click

        self.vdata.update({
            "name" : "TextButton",
            "props" : {
                "content" : self.content
            }
        })
    

    def on_event (self, event_content:dict):
        if event_content['name'] == "on_click":
            on_view_action(self.on_click, [self])
    

    def update(self):
        self.vdata.update({
            "name" : "TextButton",
            "props" : {
                "content" : self.content
            }
        })
        return super().update()


from .subviewparent import SubViewParent



class VStack (SubViewParent):
    """Align the subviews in a parent stack verticaly."""
    def __init__(self, padding:int=0) -> None:
        super().__init__()

        self.padding : int = padding

        self.vdata.update({
            "name" : "VStack",
            "props" : {
                "padding" : self.padding
            }
        })
    
    def update (self):
        self.vdata.update({
            "props" : {
                "padding" : int(self.padding)
            }
        })
        super().update()


from ..subview import SubView



class RoundedRectangleShape (SubView):
    def __init__(self, corner_radius:int=5, color:str="primery", width:int=100, height:int=100) -> None:
        super().__init__()

        self.corner_radius : int = corner_radius
        self.color : str = color
        self.width : int = width
        self.height : int = height
        
        self.vdata.update({
            "name" : " RoundedRectangleShape",
            "props" : {
                "corner_radius" : self.corner_radius,
                "color" : color,
                "width" : width,
                "height" : height
            }
        })
    

    def update(self):
        self.vdata.update({
            "name" : " RoundedRectangleShape",
            "props" : {
                "corner_radius" : self.corner_radius,
                "color" : self.color,
                "width" : self.width,
                "height" : self.height
            }
        })
        return super().update()


import subprocess
import os
import shutil
import requests
from .unzip_assets import unzip_file
from .pyinstaller_check import is_run_on_pyinstaller

def run_swiftUI_app(port, tmp_dir):
    # get the current temporary folder.
    temp_dir = tmp_dir

    # Prepare paths
    zip_path = str(__file__).replace("tools/run_swiftUI.py", "assets/swoopyui.zip")
    new_app_path = os.path.join(temp_dir, "swoopyui.app/")

    if is_run_on_pyinstaller():
        # install the zip file from github
        url = "https://raw.githubusercontent.com/SKbarbon/swoopyui/main/swoopyui/assets/swoopyui.zip"
        path_of_installed_zip = os.path.join(temp_dir, "app_zip_file")
        with open(path_of_installed_zip, "wb") as f:
            f.write(requests.get(url).content)
        zip_path = path_of_installed_zip

    # unzip the app on the temporary folder
    unzip_file(zip_path=zip_path, destination_path=temp_dir)

    # prepare the commands
    executable_of_the_app = os.path.join(new_app_path, "Contents/MacOS/swoopyui")
    chmod_command = ["chmod", "+x", executable_of_the_app]

    # run the `chmod` command
    subprocess.run(chmod_command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

    # run the command the will start the swiftUI app
    run_command = [executable_of_the_app, str(port)]
    subprocess.run(run_command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

    # Remove the temporary dir.
    if os.path.isdir (temp_dir):
        shutil.rmtree(temp_dir)